const async = require('async');
const pg = require('pg').Client;
const https = require('https');

const db_config = {
    region: process.env.AWS_REGION,
    httpOptions: {
        agent: new https.Agent({
            ciphers: 'ALL',
            secureProtocol: 'TLSv1_method'
        })
    }
};

const dynamoDB = require('aws-dynamodb')(db_config);
const table = 'fndy_object_type_' + process.env.ENV;
const limit = 500;

exports.handler = function(event, context, callback) {

    const client = new pg({
        host: process.env.DB_HOST,//'fndy-search-result.cildgunnxgsj.us-west-2.rds.amazonaws.com',
        port: 5432,
        user: 'search_user',
        password: '0bdf3d47c38a3d61b',
        database: 'AdvancedSearch'
    });

    function init(callbackInit) {
        client.connect(err => {
            if (err) {
                console.error('connection error', err.stack);
                return callbackInit('PG connect error');
            } else {

                callbackInit(null)
            }
        })
    }

    function getDeviceTypes(cbGetDeviceTypes){

        function GetLastKeys(cbGetLastKeys) {

            var last_keys = [null];

            (function recursive_call($lastKey) {

                dynamoDB
                    .table(table)
                    .select('COUNT')
                    .resume($lastKey)
                    .limit(limit)
                    .scan(function (err) {

                        if (err) {
                            return cbGetLastKeys(err);
                        }

                        if (this.LastEvaluatedKey === null) {
                            return cbGetLastKeys(null, last_keys);
                        } else {
                            var $this = this;
                            setTimeout(function () {
                                last_keys.push($this.LastEvaluatedKey);
                                recursive_call($this.LastEvaluatedKey);
                            }, 0);
                        }

                    });

            })(null);

        }

        function GetDeviceTypes(last_keys, cbGetDeviceTypes) {

            async.map(last_keys, function (last_key, cb) {

                dynamoDB
                    .table(table)
                    .resume(last_key)
                    .limit(limit)
                    .scan(function (err, data) {

                        if (err) {
                            return cb(err);
                        }
                        cb(null, data);

                    });

            }, function (err, result) {

                if (err) {
                    return cbGetDeviceTypes(err);
                }

                return cbGetDeviceTypes(null, [].concat.apply([], result));

            });

        }

        async.waterfall([
            GetLastKeys,
            GetDeviceTypes
        ], cbGetDeviceTypes);
    }

    function createTables(types, cbCreateTables){
        async.map(types, (type, cbMap)=>{
            let mandatoryFields = {
                deviceId: 'VARCHAR(100)',//'String',
                accountId: 'VARCHAR(100)',//'String',
                typeId: 'VARCHAR(100)',//'String',
                epochTime: 'BIGINT'//'Integer'
            };

            let fieldList= '';

            for (let fieldName in mandatoryFields){

                fieldList += `"${fieldName}" ${mandatoryFields[fieldName]}, `;
            }

            for (let fieldName in type){
                let typeField = type[fieldName];
                if(typeField && typeField.characteristics && Array.isArray(typeField.characteristics)) {

                    fieldList += `"${fieldName}" ${selectTypeForColumn(type[fieldName]['data_type'])}, `;
                }
            }

            let requestString = `CREATE TABLE IF NOT EXISTS "_history_type_${type.typeId}" (${fieldList.slice(0, -2)})`;

            client.query(requestString, cbMap)
        }, cbCreateTables)
    }

    function selectTypeForColumn(typeFromDynamoDB){

        switch (typeFromDynamoDB) {
            case "Integer":
                return 'BIGINT';
            case "Decimal":
                return 'NUMERIC';
            default:
                return 'TEXT';
        }
    }

    async.waterfall([
        init,
        getDeviceTypes,
        createTables
    ], function (err, result) {

        client.end();

        if (err) {
            console.error('error:', err);
            const response = {
                status: 400,
                error: err.message ? err.message : err
            };
            return context.succeed(JSON.stringify(response));
        }

        return context.succeed(result);

    });
};
/*
CREATE TABLE "_history_type_16dc7e50-e4fc-11e7-97cd-d17b4174b15e"
(
    "deviceId"             VARCHAR(50),
    "accountId"             VARCHAR(50),
    "epochTime"            BIGINT,
    "a.email_alert"      TEXT,
    "a.name"             TEXT,
    "a.reference_object" VARCHAR(50),
    "a.text_alert"       TEXT
);

CREATE TABLE "_history_type_d52284f0-e4f1-11e7-97cd-d17b4174b15e"
(
  "a.air_temp_alarm_count"            NUMERIC,
  "a.air_temperature"                 NUMERIC,
  "a.air_temperature_alarm"           NUMERIC,
  "a.alarm_status"                    VARCHAR(50),
  "a.barometric_pressure"             NUMERIC,
  "a.create_air_temperature_alarm"    NUMERIC,
  "a.datetime"                        BIGINT,
  "a.increment_air_temperature_count" NUMERIC,
  "a.lat"                             VARCHAR(50),
  "a.long"                            VARCHAR(50),
  "a.location"                        TEXT,
  "a.max_wind_speed"                  NUMERIC,
  "a.name"                            TEXT,
  "a.rainfall"                        TEXT,
  "a.relative_humidity"               NUMERIC,
  "a.temperature_celsius"             NUMERIC,
  "a.wind_direction"                  NUMERIC,
  "a.wind_speed"                      NUMERIC,
  "deviceId"                            VARCHAR(50),
  "epochTime"                           BIGINT,
  "accountId"                           VARCHAR(50),
  "a.average_temp"      NUMERIC,
  "a.barometric_pressure"   NUMERIC,
  "a.date"  TEXT,

)

*/